@@ -1,41 +1,31 @@
 /*
- *   $Id: device-common.c,v 1.3 1997/01/16 21:14:38 lf Exp $
+ *   $Id: device-common.c,v 1.3 2002/01/02 11:01:11 psavola Exp $
  *
  *   Authors:
  *    Lars Fenneberg		<lf@elemental.net>	 
  *
- *   This software is Copyright 1996 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997 by the above mentioned author(s), 
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lf@elemental.net>.
+ *   may request it from <lutchann@litech.org>.
  *
  */
 
-#include "config.h"
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <syslog.h>
-#include <errno.h>
-
-#include <sys/types.h>
-#include <sys/ioctl.h>
-#include <sys/socket.h>
-#include <netinet6/in6.h>
-#include <netinet6/ipv6.h>
-
-#include "radvd.h"
-#include "defaults.h"
+#include <config.h>
+#include <includes.h>
+#include <radvd.h>
+#include <defaults.h>
 
 int
 check_device(int sock, struct Interface *iface)
 {
 	struct ifreq	ifr;
 	
-	strcpy(ifr.ifr_name, iface->Name);
-	
+	strncpy(ifr.ifr_name, iface->Name, IFNAMSIZ-1);
+	ifr.ifr_name[IFNAMSIZ-1] = '\0';
+
 	if (ioctl(sock, SIOCGIFFLAGS, &ifr) < 0)
 	{
 		log(LOG_ERR, "ioctl(SIOCGIFFLAGS) failed for %s: %s", 
@@ -49,16 +39,18 @@ check_device(int sock, struct Interface *iface)
 		return (-1);
 	}
 	
-	if (!(ifr.ifr_flags & IFF_MULTICAST))
+	if (! iface->UnicastOnly && !(ifr.ifr_flags & IFF_MULTICAST))
 	{
 		log(LOG_WARNING, "interface %s does not support multicast",
 			iface->Name);
+		log(LOG_WARNING, "   do you need to add the UnicastOnly flag?");
 	}
 
-	if (!(ifr.ifr_flags & IFF_BROADCAST))
+	if (! iface->UnicastOnly && !(ifr.ifr_flags & IFF_BROADCAST))
 	{
 		log(LOG_WARNING, "interface %s does not support broadcast",
 			iface->Name);
+		log(LOG_WARNING, "   do you need to add the UnicastOnly flag?");
 	}
 
 	return 0;
