@@ -1,35 +1,22 @@
 /*
- *   $Id: process.c,v 1.5 1997/01/16 21:14:38 lf Exp $
+ *   $Id: process.c,v 1.5 2001/11/14 19:58:11 lutchann Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
  *    Lars Fenneberg		<lf@elemental.net>	 
  *
- *   This software is Copyright 1996 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997 by the above mentioned author(s), 
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lf@elemental.net>.
+ *   may request it from <lutchann@litech.org>.
  *
  */
 
-#include "config.h"
-
-#include <syslog.h>
-#include <string.h>
-#include <errno.h>
-#include <stdlib.h>
-
-#include <sys/types.h>
-#include <sys/time.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netinet/in.h>
-#include <netinet6/in6.h>
-#include <netinet6/nd6_protocol.h>
-
-#include "radvd.h"
+#include <config.h>
+#include <includes.h>
+#include <radvd.h>
 
 static void process_rs(int, struct Interface *, struct sockaddr_in6 *);
 static void process_ra(struct Interface *, unsigned char *msg, int len,
@@ -42,23 +29,30 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 	struct sockaddr_in6 *addr, struct in6_pktinfo *pkt_info, int hoplimit)
 {
 	struct Interface *iface;
-	struct icmpv6hdr *icmph;
+	struct icmp6_hdr *icmph;
 	char addr_str[INET6_ADDRSTRLEN];
 
+	if ( ! pkt_info )
+	{
+		log(LOG_WARNING, "received packet with no pkt_info!" );
+		return;
+	}
+
 	/*
 	 * can this happen?
 	 */
 
-	if (len < sizeof(struct icmpv6hdr)) {
+	if (len < sizeof(struct icmp6_hdr))
+	{
 		log(LOG_WARNING, "received icmpv6 packet with invalid length: %d",
 			len);
 		return;
 	}
 
-	icmph = (struct icmpv6hdr *) msg;
+	icmph = (struct icmp6_hdr *) msg;
 
-	if (icmph->icmpv6_type != ND6_ROUTER_SOLICITATION &&
-	    icmph->icmpv6_type != ND6_ROUTER_ADVERTISEMENT)
+	if (icmph->icmp6_type != ND_ROUTER_SOLICIT &&
+	    icmph->icmp6_type != ND_ROUTER_ADVERT)
 	{
 		/*
 		 *	We just want to listen to RSs and RAs
@@ -68,6 +62,41 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 		return;
 	}
 
+	if (icmph->icmp6_type == ND_ROUTER_ADVERT)
+	{
+		if (len < sizeof(struct nd_router_advert)) {
+			log(LOG_WARNING, "received icmpv6 RA packet with invalid length: %d",
+				len);
+			return;
+		}
+
+		if (!IN6_IS_ADDR_LINKLOCAL(&addr->sin6_addr)) {
+			log(LOG_WARNING, "received icmpv6 RA packet with non-linklocal source address");
+			return;
+		}
+	}			
+	
+	if (icmph->icmp6_type == ND_ROUTER_SOLICIT)
+	{
+		if (len < sizeof(struct nd_router_solicit)) {
+			log(LOG_WARNING, "received icmpv6 RS packet with invalid length: %d",
+				len);
+			return;
+		}
+		
+		/* 
+		 * XXX: RFC2461 6.1.1: if RS src addr is the unspecified address,
+		 *	and there is a lladdr option, the packet MUST be dropped
+		 */
+	}			
+
+	if (icmph->icmp6_code != 0)
+	{
+		log(LOG_WARNING, "received icmpv6 RS/RA packet with invalid code: %d",
+			icmph->icmp6_code);
+		return;
+	}
+	
 	dlog(LOG_DEBUG, 4, "if_index %d", pkt_info->ipi6_ifindex);
 
 	/* get iface by received if_index */
@@ -103,11 +132,11 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 
 	dlog(LOG_DEBUG, 4, "found Interface: %s", iface->Name);
 
-	if (icmph->icmpv6_type == ND6_ROUTER_SOLICITATION)
+	if (icmph->icmp6_type == ND_ROUTER_SOLICIT)
 	{
 		process_rs(sock, iface, addr);
 	}
-	else if (icmph->icmpv6_type == ND6_ROUTER_ADVERTISEMENT)
+	else if (icmph->icmp6_type == ND_ROUTER_ADVERT)
 	{
 		process_ra(iface, msg, len, addr);
 	}
@@ -116,16 +145,18 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 static void
 process_rs(int sock, struct Interface *iface, struct sockaddr_in6 *addr)
 {
-	int delay, next;
+	int delay;
+	double next;
 	struct timeval tv;
 
 	gettimeofday(&tv, NULL);
 
 	delay = (int) (MAX_RA_DELAY_TIME*rand()/(RAND_MAX+1.0));
-	dlog(LOG_DEBUG, 3, "random delay for %s: %d", iface->Name, delay);
+	dlog(LOG_DEBUG, 3, "random mdelay for %s: %d", iface->Name, delay);
 	mdelay(delay);
  	
-	if (tv.tv_sec - iface->last_multicast < MIN_DELAY_BETWEEN_RAS)
+	if (iface->UnicastOnly
+		|| tv.tv_sec - iface->last_multicast < MIN_DELAY_BETWEEN_RAS)
 	{
 		/*
 		 *	unicast reply
@@ -139,7 +170,7 @@ process_rs(int sock, struct Interface *iface, struct sockaddr_in6 *addr)
 		 *	multicast reply
 		 */
 
-		(void) clear_timer(&iface->tm);
+		clear_timer(&iface->tm);
 		send_ra(sock, iface, NULL);
 		
 		next = rand_between(iface->MinRtrAdvInterval, iface->MaxRtrAdvInterval); 
@@ -148,7 +179,7 @@ process_rs(int sock, struct Interface *iface, struct sockaddr_in6 *addr)
 }
 
 /*
- * check router advertisements according to RFC 1970, 6.2.7
+ * check router advertisements according to RFC 2461, 6.2.7
  */
 static void
 process_ra(struct Interface *iface, unsigned char *msg, int len, 
@@ -156,40 +187,40 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 {
 	struct nd_router_advert *radvert;
 	char addr_str[INET6_ADDRSTRLEN];
-	u_int8_t *opt_str;
+	uint8_t *opt_str;
 
 	print_addr(&addr->sin6_addr, addr_str);
 
 	radvert = (struct nd_router_advert *) msg;
 
-	if ((radvert->radv_maxhoplimit && iface->AdvCurHopLimit) && 
-	   (radvert->radv_maxhoplimit != iface->AdvCurHopLimit))
+	if ((radvert->nd_ra_curhoplimit && iface->AdvCurHopLimit) && 
+	   (radvert->nd_ra_curhoplimit != iface->AdvCurHopLimit))
 	{
 		log(LOG_WARNING, "our AdvCurHopLimit on %s doesn't agree with %s",
 			iface->Name, addr_str);
 	}
 
-	if ((radvert->radv_m_o_res & ND6_RADV_M_BIT) && !iface->AdvManagedFlag)
+	if ((radvert->nd_ra_flags_reserved & ND_RA_FLAG_MANAGED) && !iface->AdvManagedFlag)
 	{
 		log(LOG_WARNING, "our AdvManagedFlag on %s doesn't agree with %s",
 			iface->Name, addr_str);
 	}
 	
-	if ((radvert->radv_m_o_res & ND6_RADV_O_BIT) && !iface->AdvOtherConfigFlag)
+	if ((radvert->nd_ra_flags_reserved & ND_RA_FLAG_OTHER) && !iface->AdvOtherConfigFlag)
 	{
 		log(LOG_WARNING, "our AdvOtherConfigFlag on %s doesn't agree with %s",
 			iface->Name, addr_str);
 	}
 
-	if ((radvert->radv_reachable && iface->AdvReachableTime) &&
-	   (ntohl(radvert->radv_reachable) != iface->AdvReachableTime))
+	if ((radvert->nd_ra_reachable && iface->AdvReachableTime) &&
+	   (ntohl(radvert->nd_ra_reachable) != iface->AdvReachableTime))
 	{
 		log(LOG_WARNING, "our AdvReachableTime on %s doesn't agree with %s",
 			iface->Name, addr_str);
 	}
 	
-	if ((radvert->radv_retransmit && iface->AdvRetransTimer) &&
-	   (ntohl(radvert->radv_retransmit) != iface->AdvRetransTimer))
+	if ((radvert->nd_ra_retransmit && iface->AdvRetransTimer) &&
+	   (ntohl(radvert->nd_ra_retransmit) != iface->AdvRetransTimer))
 	{
 		log(LOG_WARNING, "our AdvRetransTimer on %s doesn't agree with %s",
 			iface->Name, addr_str);
@@ -200,16 +231,16 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 	if (len == 0)
 		return;
 		
-	opt_str = (u_int8_t *)(msg + sizeof(struct nd_router_advert));
+	opt_str = (uint8_t *)(msg + sizeof(struct nd_router_advert));
 		
 	while (len > 0)
 	{
 		int optlen;
-		struct nd6_opt_prefix_info *pinfo;
-		struct nd6_opt_mtu *mtu;
+		struct nd_opt_prefix_info *pinfo;
+		struct nd_opt_mtu *mtu;
 		struct AdvPrefix *prefix;
 		char prefix_str[INET6_ADDRSTRLEN];
-		u_int32_t preferred, valid;
+		uint32_t preferred, valid;
 
 		if (len < 2)
 		{
@@ -236,25 +267,26 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 
 		switch (*opt_str)
 		{
-		case ND6_OPT_MTU:
-			mtu = (struct nd6_opt_mtu *)opt_str;
+		case ND_OPT_MTU:
+			mtu = (struct nd_opt_mtu *)opt_str;
 
-			if (iface->AdvLinkMTU && (ntohl(mtu->opt_mtu) != iface->AdvLinkMTU))
+			if (iface->AdvLinkMTU && (ntohl(mtu->nd_opt_mtu_mtu) != iface->AdvLinkMTU))
 			{
 				log(LOG_WARNING, "our AdvLinkMTU on %s doesn't agree with %s",
 					iface->Name, addr_str);
 			}
 			break;
-		case ND6_OPT_PREFIX_INFORMATION:
-			pinfo = (struct nd6_opt_prefix_info *) opt_str;
-			preferred = ntohl(pinfo->opt_preferred_life);
-			valid = ntohl(pinfo->opt_valid_life);
+		case ND_OPT_PREFIX_INFORMATION:
+			pinfo = (struct nd_opt_prefix_info *) opt_str;
+			preferred = ntohl(pinfo->nd_opt_pi_preferred_time);
+			valid = ntohl(pinfo->nd_opt_pi_valid_time);
 			
 			prefix = iface->AdvPrefixList;
 			while (prefix)
 			{
-				if ((prefix->PrefixLen == pinfo->opt_prefix_length) &&
-				    addr_match(&prefix->Prefix, &pinfo->opt_prefix,
+				if (prefix->enabled &&
+				    (prefix->PrefixLen == pinfo->nd_opt_pi_prefix_len) &&
+				    addr_match(&prefix->Prefix, &pinfo->nd_opt_pi_prefix,
 				    	 prefix->PrefixLen))
 				{
 					print_addr(&prefix->Prefix, prefix_str);
@@ -282,17 +314,22 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 				prefix = prefix->next;
 			}			
 			break;
-		case ND6_OPT_SOURCE_LINKADDR:
+		case ND_OPT_SOURCE_LINKADDR:
 			/* not checked */
 			break;
-		case ND6_OPT_TARGET_LINKADDR:
-		case ND6_OPT_REDIRECTED_HEADER:
-			log(LOG_ERR, "invalid option %d in RA from %s",
-				*opt_str, addr_str);
+		case ND_OPT_TARGET_LINKADDR:
+		case ND_OPT_REDIRECTED_HEADER:
+			log(LOG_ERR, "invalid option %d in RA on %s from %s",
+				(int)*opt_str, iface->Name, addr_str);
+			break;
+		/* Mobile IPv6 extensions */
+		case ND_OPT_RTR_ADV_INTERVAL:
+		case ND_OPT_HOME_AGENT_INFO:
+			/* not checked */
 			break;
 		default:
-			dlog(LOG_DEBUG, 1, "unknown option %d in RA from %s",
-				*opt_str, addr_str);
+			dlog(LOG_DEBUG, 1, "unknown option %d in RA on %s from %s",
+				(int)*opt_str, iface->Name, addr_str);
 			break;
 		}
 		
@@ -307,8 +344,8 @@ addr_match(struct in6_addr *a1, struct in6_addr *a2, int prefixlen)
 	int pdw;
 	int pbi;
 
-	pdw = prefixlen >> 0x05;  /* num of whole u_int32_t in prefix */
-	pbi = prefixlen &  0x1f;  /* num of bits in incomplete u_int32_t in prefix */
+	pdw = prefixlen >> 0x05;  /* num of whole uint32_t in prefix */
+	pbi = prefixlen &  0x1f;  /* num of bits in incomplete uint32_t in prefix */
 
 	if (pdw) 
 	{
@@ -318,13 +355,13 @@ addr_match(struct in6_addr *a1, struct in6_addr *a2, int prefixlen)
 
 	if (pbi) 
 	{
-		u_int32_t w1, w2;
-		u_int32_t mask;
+		uint32_t w1, w2;
+		uint32_t mask;
 
-		w1 = *((u_int32_t *)a1 + pdw);
-		w2 = *((u_int32_t *)a2 + pdw);
+		w1 = *((uint32_t *)a1 + pdw);
+		w2 = *((uint32_t *)a2 + pdw);
 
-		mask = htonl(((u_int32_t) 0xffffffff) << (0x20 - pbi));
+		mask = htonl(((uint32_t) 0xffffffff) << (0x20 - pbi));
 
 		if ((w1 ^ w2) & mask)
 			return 0;
