@@ -1,37 +1,27 @@
 /*
- *   $Id: radvd.h,v 1.9 1997/01/19 17:11:29 lf Exp $
+ *   $Id: radvd.h,v 1.7 2001/11/14 19:58:11 lutchann Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
  *    Lars Fenneberg		<lf@elemental.net>	 
  *
- *   This software is Copyright 1996 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997 by the above mentioned author(s), 
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lf@elemental.net>.
+ *   may request it from <lutchann@litech.org>.
  *
  */
 
 #ifndef RADV_H
 #define RADV_H
 
-#include "config.h"
+#include <config.h>
+#include <includes.h>
+#include <defaults.h>
 
-#include <unistd.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netinet6/in6.h>
-#include <netinet6/ipv6.h>
-#include <net/if.h>
-
-/* protocol constants as specified by RFC 1970 */
-#define MAX_INITIAL_RTR_ADVERT_INTERVAL	       16 /* seconds */
-#define MAX_INITIAL_RTR_ADVERTISEMENTS		3 /* transmissions */
-#define MAX_FINAL_RTR_ADVERTISEMENTS      	3 /* transmissions */
-#define MAX_RA_DELAY_TIME                   500.0 /* milliseconds */
-#define MIN_DELAY_BETWEEN_RAS			3 /* seconds */
+#define CONTACT_EMAIL	"Nathan Lutchansky <lutchann@litech.org>"
 
 /* for log.c */
 #define	L_NONE		0
@@ -39,11 +29,10 @@
 #define L_STDERR	2
 #define L_LOGFILE	3
 
-/* maximum message size for incoming and outgoing RSs and RAs */
-#define MSG_SIZE	4096
+#define LOG_TIME_FORMAT "%b %d %H:%M:%S"
 
 struct timer_lst {
-	unsigned long		expires;
+	struct timeval		expires;
 	void			(*handler)(void *);
 	void *			data;
 	struct timer_lst	*next;
@@ -55,18 +44,19 @@ struct AdvPrefix;
 #define HWADDR_MAX 16
 
 struct Interface {
-	char			Name[IFNAMSIZ+1];	/* interface name */
+	char			Name[IFNAMSIZ];	/* interface name */
 
 	struct in6_addr		if_addr;
 	int			if_index;
 
-	u_int8_t		if_hwaddr[HWADDR_MAX];
-	int			if_tokenlen;
+	uint8_t			if_hwaddr[HWADDR_MAX];
+	int			if_hwaddr_len;
+	int			if_prefix_len;
 	int			if_maxmtu;
 
 	int			AdvSendAdvert;
-	int			MaxRtrAdvInterval;
-	int			MinRtrAdvInterval;
+	double			MaxRtrAdvInterval;
+	double			MinRtrAdvInterval;
 	int			AdvManagedFlag;
 	int			AdvOtherConfigFlag;
 	int			AdvLinkMTU;
@@ -75,11 +65,20 @@ struct Interface {
 	int			AdvCurHopLimit;
 	int			AdvDefaultLifetime;
 	int			AdvSourceLLAddress;
+	int			UnicastOnly;
+
+	/* Mobile IPv6 extensions */
+	int			AdvIntervalOpt;
+	int			AdvHomeAgentInfo;
+	int			AdvHomeAgentFlag;
+	int16_t			HomeAgentPreference;
+	uint16_t		HomeAgentLifetime;
 
 	struct AdvPrefix	*AdvPrefixList;
 	struct timer_lst	tm;
 	unsigned long		last_multicast;
 	struct Interface	*next;
+
 };
 
 struct AdvPrefix {
@@ -88,12 +87,37 @@ struct AdvPrefix {
 	
 	int			AdvOnLinkFlag;
 	int			AdvAutonomousFlag;
-	u_int32_t		AdvValidLifetime;
-	u_int32_t		AdvPreferredLifetime;
+	uint32_t		AdvValidLifetime;
+	uint32_t		AdvPreferredLifetime;
+
+	/* Mobile IPv6 extensions */
+        int                     AdvRouterAddr;
+
+	/* 6to4 extensions */
+	char			if6to4[IFNAMSIZ];
+	int			enabled;
 
 	struct AdvPrefix	*next;
 };
 
+/* Mobile IPv6 extensions */
+
+struct AdvInterval {
+	uint8_t			type;
+	uint8_t			length;
+	uint16_t		reserved;
+	uint32_t		adv_ival;
+};
+
+struct HomeAgentInfo {
+	uint8_t			type;
+	uint8_t			length;
+	uint16_t		reserved;
+	int16_t			preference;
+	uint16_t		lifetime;
+};	
+
+
 /* gram.y */
 int yyparse(void);
 
@@ -101,8 +125,8 @@ int yyparse(void);
 int yylex(void);
 
 /* timer.c */
-void set_timer(struct timer_lst *tm, int secs);
-unsigned long clear_timer(struct timer_lst *tm);
+void set_timer(struct timer_lst *tm, double);
+void clear_timer(struct timer_lst *tm);
 void init_timer(struct timer_lst *, void (*)(void *), void *); 
 
 /* log.c */
@@ -115,9 +139,12 @@ void set_debuglevel(int);
 int get_debuglevel(void);
 
 /* device.c */
-int setup_deviceinfo(int sock, struct Interface *);
-int check_device(int sock, struct Interface *);
-int setup_linklocal_addr(int sock, struct Interface *);
+int setup_deviceinfo(int, struct Interface *);
+int check_device(int, struct Interface *);
+int setup_linklocal_addr(int, struct Interface *);
+int setup_allrouters_membership(int, struct Interface *);
+int check_allrouters_membership(int, struct Interface *);
+int get_v4addr(const char *, unsigned int *);
 
 /* interface.c */
 void iface_init_defaults(struct Interface *);
@@ -135,11 +162,11 @@ void process(int sock, struct Interface *, unsigned char *, int,
 	struct sockaddr_in6 *, struct in6_pktinfo *, int);
 
 /* recv.c */
-int recv_rs_ra(int, char *, struct sockaddr_in6 *, struct in6_pktinfo **, int *);
+int recv_rs_ra(int, unsigned char *, struct sockaddr_in6 *, struct in6_pktinfo **, int *);
 
 /* util.c */
 void mdelay(int);
-int rand_between(int, int);
+double rand_between(double, double);
 void print_addr(struct in6_addr *, char *);
 
 #endif
