@@ -1,69 +1,62 @@
-
 /*
- *   $Id: recv.c,v 1.8 1997/01/19 17:11:29 lf Exp $
+ *   $Id: recv.c,v 1.5 2001/11/14 19:58:11 lutchann Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
  *    Lars Fenneberg		<lf@elemental.net>	 
  *
- *   This software is Copyright 1996 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997 by the above mentioned author(s), 
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lf@elemental.net>.
+ *   may request it from <lutchann@litech.org>.
  *
  */
 
-#include "config.h"
-
-#include <stdio.h>
-
-#include <syslog.h>
-#include <string.h>
-#include <errno.h>
-
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <sys/uio.h>
-
-#include "radvd.h"
-
-/* Linux doesn't conform to Posix in this respect */
-#ifndef CMSG_FIRSTHDR
-#ifdef CMSG_FIRST
-#define CMSG_FIRSTHDR	CMSG_FIRST
-#endif
-#endif
-
-#ifdef IPV6_HOPLIMIT
-#define CMSG_SIZE_MAX (2 * sizeof(struct cmsghdr) + \
-		       sizeof(struct in6_pktinfo) + \
-		       sizeof(int))
-#else
-#define CMSG_SIZE_MAX (sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo))
-#endif
+#include <config.h>
+#include <includes.h>
+#include <radvd.h>
 
 int
-recv_rs_ra(int sock, char *msg, struct sockaddr_in6 *addr,
+recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
                  struct in6_pktinfo **pkt_info, int *hoplimit)
 {                 
 	struct msghdr mhdr;
 	struct cmsghdr *cmsg;
 	struct iovec iov;
-	static unsigned char chdr[CMSG_SIZE_MAX];
+	static unsigned char *chdr = NULL;
+	static int chdrlen = 0;
 	int len;
-	
+	fd_set rfds;
+
+	if( ! chdr )
+	{
+		chdrlen = CMSG_SPACE(sizeof(struct in6_pktinfo)) +
+				CMSG_SPACE(sizeof(int));
+		chdr = malloc( chdrlen );
+	}
+
+	FD_ZERO( &rfds );
+	FD_SET( sock, &rfds );
+
+	if( select( sock+1, &rfds, NULL, NULL, NULL ) < 0 )
+	{
+		if (errno != EINTR)
+			log(LOG_ERR, "select: %s", strerror(errno));
+			
+		return -1;
+	}
+
 	iov.iov_len = MSG_SIZE;
-	iov.iov_base = msg;
+	iov.iov_base = (caddr_t) msg;
 
 	mhdr.msg_name = (caddr_t)addr;
 	mhdr.msg_namelen = sizeof(struct sockaddr_in6);
 	mhdr.msg_iov = &iov;
 	mhdr.msg_iovlen = 1;
 	mhdr.msg_control = (void *)chdr;
-	mhdr.msg_controllen = sizeof(chdr);
+	mhdr.msg_controllen = chdrlen;
 
 	len = recvmsg(sock, &mhdr, 0);
 
@@ -86,7 +79,7 @@ recv_rs_ra(int sock, char *msg, struct sockaddr_in6 *addr,
           {
 #ifdef IPV6_HOPLIMIT
               case IPV6_HOPLIMIT:
-                if ((cmsg->cmsg_len == sizeof(struct cmsghdr) + sizeof(int)) && 
+                if ((cmsg->cmsg_len == CMSG_LEN(sizeof(int))) && 
                     (*(int *)CMSG_DATA(cmsg) >= 0) && 
                     (*(int *)CMSG_DATA(cmsg) < 256))
                 {
@@ -100,15 +93,15 @@ recv_rs_ra(int sock, char *msg, struct sockaddr_in6 *addr,
                 }  
                 break;
 #endif /* IPV6_HOPLIMIT */
-              case IPV6_RXINFO:
-                if ((cmsg->cmsg_len == sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo)) &&
+              case IPV6_PKTINFO:
+                if ((cmsg->cmsg_len == CMSG_LEN(sizeof(struct in6_pktinfo))) &&
                     ((struct in6_pktinfo *)CMSG_DATA(cmsg))->ipi6_ifindex)
                 {
                   *pkt_info = (struct in6_pktinfo *)CMSG_DATA(cmsg);
                 }
                 else
                 {
-                  log(LOG_ERR, "received a bogus IPV6_RXINFO from the kernel! len=%d, index=%d", 
+                  log(LOG_ERR, "received a bogus IPV6_PKTINFO from the kernel! len=%d, index=%d", 
                   	cmsg->cmsg_len, ((struct in6_pktinfo *)CMSG_DATA(cmsg))->ipi6_ifindex);
                   return (-1);
                 } 
